@@ -393,3 +393,5 @@ data_composer_logger = OurLogger("Nuitka-DataComposer")
 onefile_logger = OurLogger("Nuitka-Onefile")
 tools_logger = OurLogger("Nuitka-Tools")
 wheel_logger = OurLogger("Nuitka-Wheel")
+cache_logger = OurLogger("Nuitka-Cache")
+reports_logger = OurLogger("Nuitka-Reports")
