@@ -281,6 +281,8 @@ from .PackageResourceCodes import (
     generateImportlibMetadataVersionCallCode,
     generateImportlibResourcesReadBinaryCallCode,
     generateImportlibResourcesReadTextCallCode,
+    generateOsListdirCallCode,
+    generateOsPathBasenameCallCode,
     generateOsPathExistsCallCode,
     generateOsPathIsdirCallCode,
     generateOsPathIsfileCallCode,
@@ -886,9 +888,11 @@ addExpressionDispatchDict(
         "EXPRESSION_IMPORTLIB_METADATA_ENTRY_POINTS_SINCE310_CALL": generateImportlibMetadataEntryPointsSince310CallCode,
         "EXPRESSION_IMPORTLIB_METADATA_BACKPORT_ENTRY_POINTS_CALL": generateImportlibMetadataBackportEntryPointsCallCode,
         "EXPRESSION_OS_UNAME_REF": generateImportModuleNameHardCode,
+        "EXPRESSION_OS_LISTDIR_REF": generateImportModuleNameHardCode,
         "EXPRESSION_OS_PATH_EXISTS_REF": generateImportModuleNameHardCode,
         "EXPRESSION_OS_PATH_ISFILE_REF": generateImportModuleNameHardCode,
         "EXPRESSION_OS_PATH_ISDIR_REF": generateImportModuleNameHardCode,
+        "EXPRESSION_OS_PATH_BASENAME_REF": generateImportModuleNameHardCode,
         "EXPRESSION_CTYPES_CDLL_REF": generateImportModuleNameHardCode,
         "EXPRESSION_CTYPES_CDLL_SINCE38_CALL": generateCtypesCdllCallCode,
         "EXPRESSION_CTYPES_CDLL_BEFORE38_CALL": generateCtypesCdllCallCode,
@@ -908,6 +912,8 @@ addExpressionDispatchDict(
         "EXPRESSION_OS_PATH_EXISTS_CALL": generateOsPathExistsCallCode,
         "EXPRESSION_OS_PATH_ISFILE_CALL": generateOsPathIsfileCallCode,
         "EXPRESSION_OS_PATH_ISDIR_CALL": generateOsPathIsdirCallCode,
+        "EXPRESSION_OS_PATH_BASENAME_CALL": generateOsPathBasenameCallCode,
+        "EXPRESSION_OS_LISTDIR_CALL": generateOsListdirCallCode,
         "EXPRESSION_MATCH_ARGS": generateMatchArgsCode,
         "EXPRESSION_STR_OPERATION_FORMAT": generateStrFormatMethodCode,
     }
