@@ -31,9 +31,10 @@ from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonVersions import python_version
 from nuitka.Tracing import recursion_logger
 from nuitka.utils.FileOperations import listDir
+from nuitka.utils.Importing import getSharedLibrarySuffixes
 from nuitka.utils.ModuleNames import ModuleName
 
-from .Importing import getModuleNameAndKindFromFilename, locateModule
+from .Importing import locateModule
 
 
 def _recurseTo(module_name, module_filename, module_kind):
@@ -55,7 +56,15 @@ def _recurseTo(module_name, module_filename, module_kind):
     return module, is_added
 
 
-def recurseTo(signal_change, module_name, module_filename, module_kind, reason):
+def recurseTo(
+    signal_change,
+    module_name,
+    module_filename,
+    module_kind,
+    using_module,
+    source_ref,
+    reason,
+):
     try:
         module = ImportCache.getImportedModuleByNameAndPath(
             module_name, module_filename
@@ -68,6 +77,8 @@ def recurseTo(signal_change, module_name, module_filename, module_kind, reason):
             module_filename=module_filename,
             module_name=module_name,
             module_kind=module_kind,
+            using_module=using_module,
+            source_ref=source_ref,
         )
 
         module, added_flag = _recurseTo(
@@ -82,7 +93,21 @@ def recurseTo(signal_change, module_name, module_filename, module_kind, reason):
     return module
 
 
+_recursion_decision_cache = {}
+
+
 def decideRecursion(module_filename, module_name, module_kind, extra_recursion=False):
+    key = module_filename, module_name, module_kind, extra_recursion
+
+    if key not in _recursion_decision_cache:
+        _recursion_decision_cache[key] = _decideRecursion(
+            module_filename, module_name, module_kind, extra_recursion
+        )
+
+    return _recursion_decision_cache[key]
+
+
+def _decideRecursion(module_filename, module_name, module_kind, extra_recursion):
     # Many branches, which make decisions immediately, by returning
     # pylint: disable=too-many-branches,too-many-return-statements
     if module_name == "__main__":
@@ -176,27 +201,6 @@ def decideRecursion(module_filename, module_name, module_kind, extra_recursion=F
     )
 
 
-def considerFilename(module_filename):
-    module_filename = os.path.normpath(module_filename)
-
-    if os.path.isdir(module_filename):
-        module_filename = os.path.abspath(module_filename)
-
-        module_name = os.path.basename(module_filename)
-
-        return module_filename, module_name
-    elif module_filename.endswith(".py"):
-        module_name = os.path.basename(module_filename)[:-3]
-
-        return module_filename, module_name
-    elif module_filename.endswith(".pyw"):
-        module_name = os.path.basename(module_filename)[:-4]
-
-        return module_filename, module_name
-    else:
-        return None
-
-
 def isSameModulePath(path1, path2):
     if os.path.basename(path1) == "__init__.py":
         path1 = os.path.dirname(path1)
@@ -296,6 +300,8 @@ def checkPluginSinglePath(plugin_filename, module_package):
                 module_filename=plugin_filename,
                 module_name=module_name,
                 module_kind=module_kind,
+                using_module=None,
+                source_ref=None,
                 reason=reason,
             )
 
@@ -315,27 +321,31 @@ def checkPluginSinglePath(plugin_filename, module_package):
 def checkPluginPath(plugin_filename, module_package):
     if Options.isShowInclusion():
         recursion_logger.info(
-            "Checking top level path '%s' '%s'." % (plugin_filename, module_package)
+            "Checking top level inclusion path '%s' '%s'."
+            % (plugin_filename, module_package)
         )
 
-    plugin_info = considerFilename(module_filename=plugin_filename)
+    # Files and package directories are handled here.
+    if os.path.isfile(plugin_filename) or Importing.isPackageDir(plugin_filename):
+        checkPluginSinglePath(plugin_filename, module_package=module_package)
+    # This effectively only covers files known to not be packages due to name
+    # or older Python version.
+    elif os.path.isdir(plugin_filename):
+        for sub_path, sub_filename in listDir(plugin_filename):
+            assert sub_filename != "__init__.py"
 
-    if plugin_info is not None:
-        # File or package makes a difference, handle that
-        if os.path.isfile(plugin_info[0]) or Importing.isPackageDir(plugin_info[0]):
-            checkPluginSinglePath(plugin_filename, module_package=module_package)
-        elif os.path.isdir(plugin_info[0]):
-            for sub_path, sub_filename in listDir(plugin_info[0]):
-                assert sub_filename != "__init__.py"
+            if Importing.isPackageDir(sub_path) or sub_path.endswith(".py"):
+                checkPluginSinglePath(sub_path, module_package=None)
+                continue
 
-                if Importing.isPackageDir(sub_path) or sub_path.endswith(".py"):
+            for suffix in getSharedLibrarySuffixes():
+                if sub_path.endswith(suffix):
                     checkPluginSinglePath(sub_path, module_package=None)
-        else:
-            recursion_logger.warning(
-                "Failed to include module from %r." % plugin_info[0]
-            )
+
     else:
-        recursion_logger.warning("Failed to recurse to directory %r." % plugin_filename)
+        recursion_logger.warning(
+            "Failed to include module from '%s'." % plugin_filename
+        )
 
 
 def checkPluginFilenamePattern(pattern):
@@ -357,18 +367,23 @@ def checkPluginFilenamePattern(pattern):
         checkPluginSinglePath(filename, module_package=None)
 
     if not found:
-        recursion_logger.warning("Didn't match any files against pattern %r." % pattern)
+        recursion_logger.warning(
+            "Didn't match any files against pattern '%s'." % pattern
+        )
 
 
 def _addParentPackageUsages(using_module, module_name, signal_change, source_ref):
     for parent_package_name in module_name.getParentPackageNames():
-        _parent_package_name, parent_package_filename, finding = locateModule(
-            module_name=parent_package_name, parent_package=None, level=0
-        )
+        (
+            _parent_package_name,
+            parent_package_filename,
+            package_module_kind,
+            finding,
+        ) = locateModule(module_name=parent_package_name, parent_package=None, level=0)
 
         if parent_package_filename is None:
             recursion_logger.sysexit(
-                "Error, failed to local parent package file for '%s' parent of '%s' (used by %s) module (%s)"
+                "Error, failed to locate parent package file for '%s' parent of '%s' (used by '%s') module (%s)"
                 % (
                     parent_package_name.asString(),
                     module_name.asString(),
@@ -379,10 +394,6 @@ def _addParentPackageUsages(using_module, module_name, signal_change, source_ref
 
         assert _parent_package_name == parent_package_name
 
-        _parent_package_name, package_module_kind = getModuleNameAndKindFromFilename(
-            parent_package_filename
-        )
-
         _decision, reason = decideRecursion(
             module_filename=parent_package_filename,
             module_name=parent_package_name,
@@ -394,6 +405,8 @@ def _addParentPackageUsages(using_module, module_name, signal_change, source_ref
             module_name=parent_package_name,
             module_filename=parent_package_filename,
             module_kind=package_module_kind,
+            using_module=using_module,
+            source_ref=source_ref,
             reason=reason,
         )
 
@@ -407,62 +420,59 @@ def _addParentPackageUsages(using_module, module_name, signal_change, source_ref
 
 
 def considerUsedModules(module, signal_change):
-    for (
-        used_module_name,
-        used_module_filename,
-        finding,
-        level,
-        source_ref,
-    ) in module.getUsedModules():
-        if finding == "not-found":
+    for used_module in module.getUsedModules():
+        if used_module.finding == "not-found":
             Importing.warnAbout(
                 importing=module,
-                source_ref=source_ref,
-                module_name=used_module_name,
-                level=level,
+                source_ref=used_module.source_ref,
+                module_name=used_module.module_name,
+                level=used_module.level,
             )
 
+        # Nothing was found here
+        if used_module.filename is None:
+            continue
+
         try:
-            if used_module_filename is None:
-                continue
-
-            _module_name, module_kind = getModuleNameAndKindFromFilename(
-                used_module_filename
-            )
-
             decision, reason = decideRecursion(
-                module_filename=used_module_filename,
-                module_name=used_module_name,
-                module_kind=module_kind,
+                module_filename=used_module.filename,
+                module_name=used_module.module_name,
+                module_kind=used_module.module_kind,
             )
 
             if decision:
                 _addParentPackageUsages(
                     using_module=module,
-                    module_name=used_module_name,
+                    module_name=used_module.module_name,
                     signal_change=signal_change,
-                    source_ref=source_ref,
+                    source_ref=used_module.source_ref,
                 )
 
-                used_module = recurseTo(
+                new_module = recurseTo(
                     signal_change=signal_change,
-                    module_name=used_module_name,
-                    module_filename=used_module_filename,
-                    module_kind=module_kind,
+                    module_name=used_module.module_name,
+                    module_filename=used_module.filename,
+                    module_kind=used_module.module_kind,
+                    source_ref=used_module.source_ref,
+                    using_module=module,
                     reason=reason,
                 )
 
                 addUsedModule(
-                    module=used_module,
+                    module=new_module,
                     using_module=module,
                     usage_tag="import",
                     reason=reason,
-                    source_ref=source_ref,
+                    source_ref=used_module.source_ref,
                 )
         except NuitkaForbiddenImportEncounter as e:
             recursion_logger.sysexit(
                 "Error, forbidden import of '%s' in module '%s' at '%s' encountered."
-                % (e, module.getFullName().asString(), source_ref.getAsString())
+                % (
+                    e,
+                    module.getFullName().asString(),
+                    used_module.source_ref.getAsString(),
+                )
             )
 
     try:
