@@ -50,6 +50,28 @@ class NuitkaPluginOptionsNanny(NuitkaPluginBase):
             % (full_name, option, value)
         )
 
+    def _checkSupportedVersion(self, full_name, support_info, description):
+        if support_info == "ignore":
+            return
+
+        message = "Using module (%s %s) with no support: %s" % (
+            full_name.asString(),
+            self.getPackageVersion(full_name),
+            description,
+        )
+
+        if support_info == "error":
+            self.sysexit(message)
+        elif support_info == "warning":
+            self.warning(message)
+        elif support_info == "info":
+            self.info(message)
+        else:
+            self.sysexit(
+                "Error, unknown support_info level '%s' for module '%s'"
+                % full_name.asString()
+            )
+
     def _checkConsoleMode(self, full_name, console):
         if console == "no":
             if shallDisableConsoleWindow() is not True:
@@ -63,13 +85,15 @@ class NuitkaPluginOptionsNanny(NuitkaPluginBase):
             if shallDisableConsoleWindow() is None:
                 if isMacOS():
                     downside_message = """\
-Otherwise high resolution will not be available and a terminal window will open."""
+Otherwise high resolution will not be available and a terminal window will open"""
                 else:
                     downside_message = """\
-Otherwise a terminal window will open."""
+Otherwise a terminal window will open"""
 
                 self.info(
-                    "Note, when using '%s', consider using '--disable-console' option. %s"
+                    """\
+Note, when using '%s', consider using '--disable-console' option. %s. However for \
+debugging, terminal output is the easiest way to see informative traceback information."""
                     % (full_name, downside_message)
                 )
 
@@ -117,21 +141,27 @@ Error, package '%s' requires '--onefile' to be used on top of '--macos-create-ap
                 if self.evaluateCondition(
                     full_name=full_name, condition=check.get("when", "True")
                 ):
+                    self._checkSupportedVersion(
+                        full_name=full_name,
+                        support_info=check.get("support_info", "ignore"),
+                        description=check.get("description", "not given"),
+                    )
+
                     if mayDisableConsoleWindow():
                         self._checkConsoleMode(
                             full_name=full_name,
-                            console=options_config.get("console", "yes"),
+                            console=check.get("console", "yes"),
                         )
 
                     if isMacOS():
                         self._checkMacOSBundleMode(
                             full_name=full_name,
-                            macos_bundle=options_config.get("macos_bundle", "no"),
+                            macos_bundle=check.get("macos_bundle", "no"),
                         )
 
                         self._checkMacOSBundleOnefileMode(
                             full_name=full_name,
-                            macos_bundle_as_onefile=options_config.get(
+                            macos_bundle_as_onefile=check.get(
                                 "macos_bundle_as_onefile", "no"
                             ),
                         )
